#+linux
(define "_GNU_SOURCE")

;; largefile support on linux
;; TODO: check if these flags are required on solaris too
#+linux (define "_LARGEFILE_SOURCE")
#+linux (define "_LARGEFILE64_SOURCE")
#+linux (define "_FILE_OFFSET_BITS" 64)

(include  "sys/types.h" "sys/stat.h" "sys/socket.h" "sys/un.h"
          "netinet/in.h" "netinet/in_systm.h" "netinet/ip.h"
          "net/if.h" "netinet/tcp.h" "sys/mman.h" "sys/wait.h"
          "fcntl.h" "netdb.h" "errno.h" "signal.h" "unistd.h"
          "termios.h" "netinet/ip.h" "sys/ioctl.h" "limits.h"
          "sys/uio.h" "arpa/inet.h" "time.h" "sys/select.h"
          "sys/poll.h" "dirent.h" "pwd.h" "grp.h"
          "syslog.h" "sys/resource.h" "stdlib.h" "sys/utsname.h"
          "sys/statvfs.h"
          #+linux "sys/epoll.h"
          ;; for kqueue
          #+freebsd "sys/event.h" #+freebsd "sys/time.h")

(in-package :iolib-posix)

(ctype size "size_t")
(ctype ssize "ssize_t")
(ctype pid "pid_t")
(ctype uid "uid_t")
(ctype gid "gid_t")
(ctype off "off_t")
(ctype mode "mode_t")

(constantenum errno-values
 ((:eperm "EPERM"))
 ((:enoent "ENOENT"))
 ((:esrch "ESRCH"))
 ((:eintr "EINTR"))
 ((:eio "EIO"))
 ((:enxio "ENXIO"))
 ((:e2big "E2BIG"))
 ((:enoexec "ENOEXEC"))
 ((:ebadf "EBADF"))
 ((:echild "ECHILD"))
 ((:eagain "EAGAIN"))
 ((:enomem "ENOMEM"))
 ((:eacces "EACCES"))
 ((:efault "EFAULT"))
 ((:enotblk "ENOTBLK"))
 ((:ebusy "EBUSY"))
 ((:eexist "EEXIST"))
 ((:exdev "EXDEV"))
 ((:enodev "ENODEV"))
 ((:enotdir "ENOTDIR"))
 ((:eisdir "EISDIR"))
 ((:einval "EINVAL"))
 ((:enfile "ENFILE"))
 ((:emfile "EMFILE"))
 ((:enotty "ENOTTY"))
 ((:etxtbsy "ETXTBSY"))
 ((:efbig "EFBIG"))
 ((:enospc "ENOSPC"))
 ((:espipe "ESPIPE"))
 ((:erofs "EROFS"))
 ((:emlink "EMLINK"))
 ((:epipe "EPIPE"))
 ((:edom "EDOM"))
 ((:erange "ERANGE"))
 ((:edeadlk "EDEADLK"))
 ((:enametoolong "ENAMETOOLONG"))
 ((:enolck "ENOLCK"))
 ((:enosys "ENOSYS"))
 ((:enotempty "ENOTEMPTY"))
 ((:eloop "ELOOP"))
 ((:ewouldblock "EWOULDBLOCK"))
 ((:enomsg "ENOMSG"))
 ((:eidrm "EIDRM"))
 ((:echrng "ECHRNG") :optional t)
 ((:el2nsync "EL2NSYNC") :optional t)
 ((:el3hlt "EL3HLT") :optional t)
 ((:el3rst "EL3RST") :optional t)
 ((:elnrng "ELNRNG") :optional t)
 ((:eunatch "EUNATCH") :optional t)
 ((:enocsi "ENOCSI") :optional t)
 ((:el2hlt "EL2HLT") :optional t)
 ((:ebade "EBADE") :optional t)
 ((:ebadr "EBADR") :optional t)
 ((:exfull "EXFULL") :optional t)
 ((:enoano "ENOANO") :optional t)
 ((:ebadrqc "EBADRQC") :optional t)
 ((:ebadslt "EBADSLT") :optional t)
 ((:edeadlock "EDEADLOCK") :optional t)
 ((:ebfont "EBFONT") :optional t)
 ((:enostr "ENOSTR") :optional t)
 ((:enodata "ENODATA") :optional t)
 ((:etime "ETIME") :optional t)
 ((:enosr "ENOSR") :optional t)
 ((:enonet "ENONET") :optional t)
 ((:enopkg "ENOPKG") :optional t)
 ((:eremote "EREMOTE"))
 ((:enolink "ENOLINK"))
 ((:eadv "EADV") :optional t)
 ((:esrmnt "ESRMNT") :optional t)
 ((:ecomm "ECOMM") :optional t)
 ((:eproto "EPROTO"))
 ((:emultihop "EMULTIHOP"))
 ((:edotdot "EDOTDOT") :optional t)
 ((:ebadmsg "EBADMSG"))
 ((:eoverflow "EOVERFLOW"))
 ((:enotuniq "ENOTUNIQ") :optional t)
 ((:ebadfd "EBADFD") :optional t)
 ((:eremchg "EREMCHG") :optional t)
 ((:elibacc "ELIBACC") :optional t)
 ((:elibbad "ELIBBAD") :optional t)
 ((:elibscn "ELIBSCN") :optional t)
 ((:elibmax "ELIBMAX") :optional t)
 ((:elibexec "ELIBEXEC") :optional t)
 ((:eilseq "EILSEQ"))
 ((:erestart "ERESTART") :optional t)
 ((:estrpipe "ESTRPIPE") :optional t)
 ((:eusers "EUSERS"))
 ((:enotsock "ENOTSOCK"))
 ((:edestaddrreq "EDESTADDRREQ"))
 ((:emsgsize "EMSGSIZE"))
 ((:eprototype "EPROTOTYPE"))
 ((:enoprotoopt "ENOPROTOOPT"))
 ((:eprotonosupport "EPROTONOSUPPORT"))
 ((:esocktnosupport "ESOCKTNOSUPPORT"))
 ((:eopnotsupp "EOPNOTSUPP"))
 ((:epfnosupport "EPFNOSUPPORT"))
 ((:eafnosupport "EAFNOSUPPORT"))
 ((:eaddrinuse "EADDRINUSE"))
 ((:eaddrnotavail "EADDRNOTAVAIL"))
 ((:enetdown "ENETDOWN"))
 ((:enetunreach "ENETUNREACH"))
 ((:enetreset "ENETRESET"))
 ((:econnaborted "ECONNABORTED"))
 ((:econnreset "ECONNRESET"))
 ((:enobufs "ENOBUFS"))
 ((:eisconn "EISCONN"))
 ((:enotconn "ENOTCONN"))
 ((:eshutdown "ESHUTDOWN"))
 ((:etoomanyrefs "ETOOMANYREFS"))
 ((:etimedout "ETIMEDOUT"))
 ((:econnrefused "ECONNREFUSED"))
 ((:ehostdown "EHOSTDOWN"))
 ((:ehostunreach "EHOSTUNREACH"))
 ((:ealready "EALREADY"))
 ((:einprogress "EINPROGRESS"))
 ((:estale "ESTALE"))
 ((:euclean "EUCLEAN") :optional t)
 ((:enotnam "ENOTNAM") :optional t)
 ((:enavail "ENAVAIL") :optional t)
 ((:eremoteio "EREMOTEIO") :optional t)
 ((:edquot "EDQUOT"))
 ((:enomedium "ENOMEDIUM") :optional t)
 ((:emediumtype "EMEDIUMTYPE") :optional t))

(constant (sighup "SIGHUP") :documentation "terminal line hangup.")
(constant (sigint "SIGINT") :documentation "interrupt program.")
(constant (sigquit "SIGQUIT") :documentation "quit program.")
(constant (sigill "SIGILL") :documentation "illegal instruction.")
(constant (sigtrap "SIGTRAP") :documentation "trace trap.")
(constant (sigabrt "SIGABRT") :documentation "abort program (formerly SIGIOT).")
#-linux
(constant (sigemt "SIGEMT") :documentation "emulate instruction executed.")
(constant (sigfpe "SIGFPE") :documentation "floating-point exception.")
(constant (sigkill "SIGKILL") :documentation "kill program.")
(constant (sigbus "SIGBUS") :documentation "bus error.")
(constant (sigsegv "SIGSEGV") :documentation "segmentation violation.")
(constant (sigsys "SIGSYS") :documentation "non-existent system call invoked.")
(constant (sigpipe "SIGPIPE") :documentation "write on a pipe with no reader.")
(constant (sigalrm "SIGALRM") :documentation "real-timeimer expired.")
(constant (sigterm "SIGTERM") :documentation "softwareermination signal.")
(constant (sigurg "SIGURG") :documentation "urgent condition present on socket.")
(constant (sigstop "SIGSTOP") :documentation "stop (cannot be caught or ignored).")
(constant (sigtstp "SIGTSTP") :documentation "stop signal generated from keyboard.")
(constant (sigcont "SIGCONT") :documentation "continue after stop.")
(constant (sigchld "SIGCHLD") :documentation "child status has changed.")
(constant (sigttin "SIGTTIN") :documentation "background read attempted from controlerminal.")
(constant (sigttou "SIGTTOU") :documentation "background write attemptedo controlerminal.")
(constant (sigio "SIGIO") :documentation "I/O is possible on a descriptor (see fcntl(2)).")
(constant (sigxcpu "SIGXCPU") :documentation "cpuime limit exceeded (see setrlimit(2)).")
(constant (sigxfsz "SIGXFSZ") :documentation "file size limit exceeded (see setrlimit(2)).")
(constant (sigvtalrm "SIGVTALRM") :documentation "virtualime alarm (see setitimer(2)).")
(constant (sigprof "SIGPROF") :documentation "profilingimer alarm (see setitimer(2)).")
(constant (sigwinch "SIGWINCH") :documentation "Window size change.")
#-linux
(constant (siginfo "SIGINFO") :documentation "status request from keyboard.")
(constant (sigusr1 "SIGUSR1") :documentation "User defined signal 1.")
(constant (sigusr2 "SIGUSR2") :documentation "User defined signal 2.")
#+linux
(constant (sigrtmin "SIGRTMIN") :documentation "Smallest real-time signal number.")
#+linux
(constant (sigrtmax "SIGRTMAX") :documentation "Largest real-time signal number.")

 ;; open()
(constant (o-rdonly "O_RDONLY"))
(constant (o-wronly "O_WRONLY"))
(constant (o-rdwr "O_RDWR"))
(constant (o-creat "O_CREAT"))
(constant (o-excl "O_EXCL"))
(constant (o-noctty "O_NOCTTY"))
(constant (o-trunc "O_TRUNC"))
(constant (o-append "O_APPEND"))
(constant (o-nonblock "O_NONBLOCK"))
(constant (o-ndelay "O_NDELAY"))
(constant (o-sync "O_SYNC"))
(constant (o-nofollow "O_NOFOLLOW"))
#+linux
(constant (o-directory "O_DIRECTORY"))
(constant (o-direct "O_DIRECT"))
(constant (o-async "O_ASYNC"))
#+linux
(constant (o-largefile "O_LARGEFILE"))
#+linux
(constant (o-dsync "O_DSYNC"))
#+linux
(constant (o-rsync "O_RSYNC"))

;; fcntl()
(constant (f-dupfd "F_DUPFD"))
(constant (f-getfd "F_GETFD"))
(constant (f-setfd "F_SETFD"))
(constant (f-getfl "F_GETFL"))
(constant (f-setfl "F_SETFL"))
(constant (f-getlk "F_GETLK"))
(constant (f-setlk "F_SETLK"))
(constant (f-setlkw "F_SETLKW"))
(constant (f-getown "F_GETOWN"))
(constant (f-setown "F_SETOWN"))
#+linux
(constant (f-getsig "F_GETSIG"))
#+linux
(constant (f-setsig "F_SETSIG"))

#+linux
(constant (f-setlease "F_SETLEASE"))
#+linux
(constant (f-getlease "F_GETLEASE"))
(constant (f-rdlck "F_RDLCK"))
(constant (f-wrlck "F_WRLCK"))
(constant (f-unlck "F_UNLCK"))

;; lseek()
(constant (seek-set "SEEK_SET"))
(constant (seek-cur "SEEK_CUR"))
(constant (seek-end "SEEK_END"))

 ;; access()
(constant (r-ok "R_OK"))
(constant (w-ok "W_OK"))
(constant (x-ok "X_OK"))
(constant (f-ok "F_OK"))

;; lockf()
(constant (f-lock "F_LOCK"))
(constant (f-tlock "F_TLOCK"))
(constant (f-ulock "F_ULOCK"))
(constant (f-test "F_TEST"))

;; mmap()
(constant (prot-none "PROT_NONE") :documentation "mmap: no protection")
(constant (prot-read "PROT_READ") :documentation "mmap: read protection")
(constant (prot-write "PROT_WRITE") :documentation "mmap: write protection")
(constant (prot-exec "PROT_EXEC") :documentation "mmap: execute protection")
(constant (map-shared "MAP_SHARED") :documentation "mmap: shared memory")
(constant (map-private "MAP_PRIVATE") :documentation "mmap: private mapping")
(constant (map-fixed "MAP_FIXED") :documentation "mmap: map at location")
(constant (map-failed "MAP_FAILED") :documentation "mmap: failure")

#+linux (constant (map-noreserve "MAP_NORESERVE"))
#+linux (constant (map-locked "MAP_LOCKED"))
#+linux (constant (map-growsdown "MAP_GROWSDOWN"))
#+linux (constant (map-anon "MAP_ANON"))
#+linux (constant (map-anonymous "MAP_ANONYMOUS"))
#+linux (constant (map-32bit "MAP_32BIT"))
#+linux (constant (map-populate "MAP_POPULATE"))
#+linux (constant (map-nonblock "MAP_NONBLOCK"))

#+freebsd (constant (map-anon "MAP_ANON"))
#+freebsd (constant (map-hassemaphore "MAP_HASSEMAPHORE"))
#+freebsd (constant (map-inherit "MAP_INHERIT"))
#+freebsd (constant (map-nocore "MAP_NOCORE"))
#+freebsd (constant (map-nosync "MAP_NOSYNC"))
#+freebsd (constant (map-stack "MAP_STACK"))

;; msync()
(constant (ms-async "MS_ASYNC") :documentation "msync: return immediately")
(constant (ms-sync "MS_SYNC") :documentation "msync: perform synchronous writes")
(constant (ms-invalidate "MS_INVALIDATE") :documentation "msync: invalidate all data")

;; mlockall()
(constant (mcl-current "MCL_CURRENT") :documentation "mlockall: lock currently mapped pages.")
(constant (mcl-future "MCL_FUTURE") :documentation "mlockall: lock pages that become mapped.")

;; stat()
(constant (s-irwxu "S_IRWXU")
	  :documentation "read, write, execute/search by owner")
(constant (s-irusr "S_IRUSR")
	  :documentation "read permission, owner")
(constant (s-iwusr "S_IWUSR")
	  :documentation "write permission, owner")
(constant (s-ixusr "S_IXUSR")
	  :documentation "execute/search permission, owner")
(constant (s-irwxg "S_IRWXG")
	  :documentation "read, write, execute/search by group") 
(constant (s-irgrp "S_IRGRP")
	  :documentation "read permission, group") 
(constant (s-iwgrp "S_IWGRP")
	  :documentation "write permission, group") 
(constant (s-ixgrp "S_IXGRP")
	  :documentation "execute/search permission, group") 
(constant (s-irwxo "S_IRWXO")
	  :documentation "read, write, execute/search by others") 
(constant (s-iroth "S_IROTH")
	  :documentation "read permission, others")
(constant (s-iwoth "S_IWOTH")
	  :documentation "write permission, others")
(constant (s-ixoth "S_IXOTH")
	  :documentation "execute/search permission, others")
(constant (s-isuid "S_ISUID")
	  :documentation "set-user-ID on execution")
(constant (s-isgid "S_ISGID")
	  :documentation "set-group-ID on execution")
(constant (s-isvtx "S_ISVTX")
	  :documentation "'sticky' bit, many meanings, nonportable")
(constant (s-ifmt "S_IFMT")
	  :documentation "bitmask for type of entry")
(constant (s-ififo "S_IFIFO")
	  :documentation "named pipe, aka fifo")
(constant (s-ifchr "S_IFCHR")
	  :documentation "special character-device")
(constant (s-ifdir "S_IFDIR")
	  :documentation "directory")
(constant (s-ifblk "S_IFBLK")
	  :documentation "special block-device")
(constant (s-ifreg "S_IFREG")
	  :documentation "regular file")
(constant (s-iflnk "S_IFLNK")
	  :documentation "symbolic link")
(constant (s-ifsock "S_IFSOCK")
	  :documentation "socket")
(constant (s-ifwht "S_IFWHT")
	  :documentation "whiteout" :optional t)
(constant (s-iread "S_IREAD"))
(constant (s-iwrite "S_IWRITE"))
(constant (s-iexec "S_IEXEC"))

(constant (path-max "PATH_MAX" "MAXPATHLEN"))

;; sysconf constants
(constant (sc-aio-listio-max "_SC_AIO_LISTIO_MAX"))
(constant (sc-aio-max "_SC_AIO_MAX"))
(constant (sc-aio-prio-delta-max "_SC_AIO_PRIO_DELTA_MAX"))
(constant (sc-arg-max "_SC_ARG_MAX"))
(constant (sc-atexit-max "_SC_ATEXIT_MAX"))
(constant (sc-bc-base-max "_SC_BC_BASE_MAX"))
(constant (sc-bc-dim-max "_SC_BC_DIM_MAX"))
(constant (sc-bc-scale-max "_SC_BC_SCALE_MAX"))
(constant (sc-bc-string-max "_SC_BC_STRING_MAX"))
(constant (sc-child-max "_SC_CHILD_MAX"))
(constant (sc-clk-tck "_SC_CLK_TCK"))
(constant (sc-coll-weights-max "_SC_COLL_WEIGHTS_MAX"))
(constant (sc-delaytimer-max "_SC_DELAYTIMER_MAX"))
(constant (sc-expr-nest-max "_SC_EXPR_NEST_MAX"))
(constant (sc-host-name-max "_SC_HOST_NAME_MAX"))
(constant (sc-iov-max "_SC_IOV_MAX"))
(constant (sc-line-max "_SC_LINE_MAX"))
(constant (sc-login-name-max "_SC_LOGIN_NAME_MAX"))
(constant (sc-ngroups-max "_SC_NGROUPS_MAX"))
(constant (sc-getgr-r-size-max "_SC_GETGR_R_SIZE_MAX"))
(constant (sc-getpw-r-size-max "_SC_GETPW_R_SIZE_MAX"))
(constant (sc-mq-open-max "_SC_MQ_OPEN_MAX"))
(constant (sc-mq-prio-max "_SC_MQ_PRIO_MAX"))
(constant (sc-open-max "_SC_OPEN_MAX"))
(constant (sc-advisory-info "_SC_ADVISORY_INFO"))
(constant (sc-barriers "_SC_BARRIERS"))
(constant (sc-asynchronous-io "_SC_ASYNCHRONOUS_IO"))
(constant (sc-clock-selection "_SC_CLOCK_SELECTION"))
(constant (sc-cputime "_SC_CPUTIME"))
(constant (sc-file-locking "_SC_FILE_LOCKING"))
(constant (sc-fsync "_SC_FSYNC"))
(constant (sc-ipv6 "_SC_IPV6"))
(constant (sc-job-control "_SC_JOB_CONTROL"))
(constant (sc-mapped-files "_SC_MAPPED_FILES"))
(constant (sc-memlock "_SC_MEMLOCK"))
(constant (sc-memlock-range "_SC_MEMLOCK_RANGE"))
(constant (sc-memory-protection "_SC_MEMORY_PROTECTION"))
(constant (sc-message-passing "_SC_MESSAGE_PASSING"))
(constant (sc-monotonic-clock "_SC_MONOTONIC_CLOCK"))
(constant (sc-multi-process "_SC_MULTI_PROCESS"))
(constant (sc-prioritized-io "_SC_PRIORITIZED_IO"))
(constant (sc-priority-scheduling "_SC_PRIORITY_SCHEDULING"))
(constant (sc-raw-sockets "_SC_RAW_SOCKETS"))
(constant (sc-reader-writer-locks "_SC_READER_WRITER_LOCKS"))
(constant (sc-realtime-signals "_SC_REALTIME_SIGNALS"))
(constant (sc-regexp "_SC_REGEXP"))
(constant (sc-saved-ids "_SC_SAVED_IDS"))
(constant (sc-semaphores "_SC_SEMAPHORES"))
(constant (sc-shared-memory-objects "_SC_SHARED_MEMORY_OBJECTS"))
(constant (sc-shell "_SC_SHELL"))
(constant (sc-spawn "_SC_SPAWN"))
(constant (sc-spin-locks "_SC_SPIN_LOCKS"))
(constant (sc-sporadic-server "_SC_SPORADIC_SERVER"))
(constant (sc-symloop-max "_SC_SYMLOOP_MAX"))
(constant (sc-synchronized-io "_SC_SYNCHRONIZED_IO"))
(constant (sc-thread-attr-stackaddr "_SC_THREAD_ATTR_STACKADDR"))
(constant (sc-thread-attr-stacksize "_SC_THREAD_ATTR_STACKSIZE"))
(constant (sc-thread-cputime "_SC_THREAD_CPUTIME"))
(constant (sc-thread-prio-inherit "_SC_THREAD_PRIO_INHERIT"))
(constant (sc-thread-prio-protect "_SC_THREAD_PRIO_PROTECT"))
(constant (sc-thread-priority-scheduling "_SC_THREAD_PRIORITY_SCHEDULING"))
(constant (sc-thread-process-shared "_SC_THREAD_PROCESS_SHARED"))
(constant (sc-thread-safe-functions "_SC_THREAD_SAFE_FUNCTIONS"))
(constant (sc-thread-sporadic-server "_SC_THREAD_SPORADIC_SERVER"))
(constant (sc-threads "_SC_THREADS"))
(constant (sc-timeouts "_SC_TIMEOUTS"))
(constant (sc-timers "_SC_TIMERS"))
(constant (sc-trace "_SC_TRACE"))
(constant (sc-trace-event-filter "_SC_TRACE_EVENT_FILTER"))
(constant (sc-trace-inherit "_SC_TRACE_INHERIT"))
(constant (sc-trace-log "_SC_TRACE_LOG"))
(constant (sc-typed-memory-objects "_SC_TYPED_MEMORY_OBJECTS"))
(constant (sc-version "_SC_VERSION"))
(constant (sc-v6-ilp32-off32 "_SC_V6_ILP32_OFF32"))
(constant (sc-v6-ilp32-offbig "_SC_V6_ILP32_OFFBIG"))
(constant (sc-v6-lp64-off64 "_SC_V6_LP64_OFF64"))
(constant (sc-v6-lpbig-offbig "_SC_V6_LPBIG_OFFBIG"))
(constant (sc-2-c-bind "_SC_2_C_BIND"))
(constant (sc-2-c-dev "_SC_2_C_DEV"))
(constant (sc-2-c-version "_SC_2_C_VERSION"))
(constant (sc-2-char-term "_SC_2_CHAR_TERM"))
(constant (sc-2-fort-dev "_SC_2_FORT_DEV"))
(constant (sc-2-fort-run "_SC_2_FORT_RUN"))
(constant (sc-2-localedef "_SC_2_LOCALEDEF"))
(constant (sc-2-pbs "_SC_2_PBS"))
(constant (sc-2-pbs-accounting "_SC_2_PBS_ACCOUNTING"))
(constant (sc-2-pbs-checkpoint "_SC_2_PBS_CHECKPOINT"))
(constant (sc-2-pbs-locate "_SC_2_PBS_LOCATE"))
(constant (sc-2-pbs-message "_SC_2_PBS_MESSAGE"))
(constant (sc-2-pbs-track "_SC_2_PBS_TRACK"))
(constant (sc-2-sw-dev "_SC_2_SW_DEV"))
(constant (sc-2-upe "_SC_2_UPE"))
(constant (sc-2-version "_SC_2_VERSION"))
(constant (sc-regex-version "_SC_REGEX_VERSION"))
(constant (sc-page-size "_SC_PAGE_SIZE"))
(constant (sc-pagesize "_SC_PAGESIZE"))
(constant (sc-thread-destructor-iterations "_SC_THREAD_DESTRUCTOR_ITERATIONS"))
(constant (sc-thread-keys-max "_SC_THREAD_KEYS_MAX"))
(constant (sc-thread-stack-min "_SC_THREAD_STACK_MIN"))
(constant (sc-thread-threads-max "_SC_THREAD_THREADS_MAX"))
(constant (sc-re-dup-max "_SC_RE_DUP_MAX"))
(constant (sc-rtsig-max "_SC_RTSIG_MAX"))
(constant (sc-sem-nsems-max "_SC_SEM_NSEMS_MAX"))
(constant (sc-sem-value-max "_SC_SEM_VALUE_MAX"))
(constant (sc-sigqueue-max "_SC_SIGQUEUE_MAX"))
(constant (sc-stream-max "_SC_STREAM_MAX"))
(constant (sc-symloop-max "_SC_SYMLOOP_MAX"))
(constant (sc-timer-max "_SC_TIMER_MAX"))
(constant (sc-tty-name-max "_SC_TTY_NAME_MAX"))
(constant (sc-tzname-max "_SC_TZNAME_MAX"))
(constant (sc-xbs5-ilp32-off32 "_SC_XBS5_ILP32_OFF32"))
(constant (sc-xbs5-ilp32-offbig "_SC_XBS5_ILP32_OFFBIG"))
(constant (sc-xbs5-lp64-off64 "_SC_XBS5_LP64_OFF64"))
(constant (sc-xbs5-lpbig-offbig "_SC_XBS5_LPBIG_OFFBIG"))
(constant (sc-xopen-crypt "_SC_XOPEN_CRYPT"))
(constant (sc-xopen-enh-i18n "_SC_XOPEN_ENH_I18N"))
(constant (sc-xopen-legacy "_SC_XOPEN_LEGACY"))
(constant (sc-xopen-realtime "_SC_XOPEN_REALTIME"))
(constant (sc-xopen-realtime-threads "_SC_XOPEN_REALTIME_THREADS"))
(constant (sc-xopen-shm "_SC_XOPEN_SHM"))
(constant (sc-xopen-streams "_SC_XOPEN_STREAMS"))
(constant (sc-xopen-unix "_SC_XOPEN_UNIX"))
(constant (sc-xopen-version "_SC_XOPEN_VERSION"))
(constant (sc-xopen-xcu-version "_SC_XOPEN_XCU_VERSION"))

;;
;; from sys/uio.h
;;
(cstruct iovec "struct iovec"
  (base "iov_base" :type :pointer)
  (len  "iov_len"  :type size))
(constant (iov-max #+freebsd "IOV_MAX"
                   #+linux "UIO_MAXIOV"))

;;
;; from sys/socket.h
;;
(ctype socklen   "socklen_t")
(ctype sa-family "sa_family_t")

;; socket() - socket address family
(constant (af-unspec "AF_UNSPEC" "PF_UNSPEC"))
(constant (af-inet "AF_INET" "PF_INET") :documentation "IPv4 Protocol family")
(constant (af-inet6 "AF_INET6" "PF_INET6") :documentation "IPv6 Protocol family")
(constant (af-local "AF_UNIX" "AF_LOCAL" "PF_UNIX" "PF_LOCAL")
  :documentation "File domain sockets")
(constant (af-packet "AF_PACKET" "PF_PACKET") :documentation "Raw packet access"
  :optional t)
(constant (af-route "AF_ROUTE" "PF_ROUTE") :documentation "Routing sockets" :optional t)
(constant (af-key "AF_KEY" "PF_KEY"))
(constant (af-netlink "AF_NETLINK" "PF_NETLINK") :documentation "Linux Netlink sockets"
  :optional t)

;; socket() - socket type
(constant (sock-stream "SOCK_STREAM") :documentation "TCP")
(constant (sock-dgram "SOCK_DGRAM") :documentation "UDP")
(constant (sock-seqpacket "SOCK_SEQPACKET")
  :documentation "Reliable Sequenced Datagram Protocol" :optional t)
(constant (sock-raw "SOCK_RAW") :documentation "Raw protocol access" :optional t)
(constant (sock-rdm "SOCK_RDM")
  :documentation "Reliable Unordered Datagram Protocol" :optional t)

;; socket() - socket protocol
(constant (ipproto-ip "IPPROTO_IP"))
(constant (ipproto-ipv6 "IPPROTO_IPV6"))
(constant (ipproto-icmp "IPPROTO_ICMP"))
(constant (ipproto-icmpv6 "IPPROTO_ICMPV6"))
(constant (ipproto-raw "IPPROTO_RAW"))
(constant (ipproto-tcp "IPPROTO_TCP"))
(constant (ipproto-udp "IPPROTO_UDP"))
(constant (ipproto-sctp "IPPROTO_SCTP"))

(cstruct sockaddr "struct sockaddr"  
  (family "sa_family" :type sa-family))

(cstruct sockaddr-storage "struct sockaddr_storage"
  (family "ss_family" :type sa-family))

(constant (somaxconn "SOMAXCONN")
  :documentation "Maximum listen() queue length")
   
(constant (sol-socket "SOL_SOCKET")
  :documentation "get/setsockopt socket level constant.")
#+linux
(constant (sol-tcp "SOL_TCP")
  :documentation "get/setsockopt TCP level constant.")
#+linux
(constant (sol-ip "SOL_IP")
  :documentation "get/setsockopt IP level constant.")
#+linux
(constant (sol-raw "SOL_RAW")
  :documentation "get/setsockopt raw level constant.")

;; getsockopt/setsockopt()
(constant (so-acceptconn "SO_ACCEPTCONN"))
(constant (so-bsdcompat "SO_BSDCOMPAT") :optional t)       ; linux
(constant (so-bindtodevice "SO_BINDTODEVICE") :optional t) ; linux
(constant (so-broadcast "SO_BROADCAST"))
(constant (so-debug "SO_DEBUG"))
(constant (so-dontroute "SO_DONTROUTE"))
(constant (so-error "SO_ERROR"))
(constant (so-keepalive "SO_KEEPALIVE"))
(constant (so-linger "SO_LINGER"))
(constant (so-oobinline "SO_OOBINLINE"))
(constant (so-passcred "SO_PASSCRED") :optional t) ; linux
(constant (so-peercred "SO_PEERCRED") :optional t) ; linux
(constant (so-priority "SO_PRIORITY") :optional t) ; linux
(constant (so-rcvbuf "SO_RCVBUF"))
(constant (so-rcvlowat "SO_RCVLOWAT"))
(constant (so-rcvtimeo "SO_RCVTIMEO"))
(constant (so-reuseaddr "SO_REUSEADDR"))
(constant (so-sndbuf "SO_SNDBUF"))
(constant (so-sndlowat "SO_SNDLOWAT"))
(constant (so-sndtimeo "SO_SNDTIMEO"))
(constant (so-timestamp "SO_TIMESTAMP"))
(constant (so-type "SO_TYPE"))
(constant (so-useloopback "SO_USELOOPBACK") :optional t)      ; freebsd
(constant (so-reuseport "SO_REUSEPORT") :optional t)          ; freebsd
(constant (so-nosigpipe "SO_NOSIGPIPE") :optional t)          ; freebsd
(constant (so-acceptfilter "SO_ACCEPTFILTER") :optional t)    ; freebsd
(constant (so-bintime "SO_BINTIME") :optional t)              ; freebsd
(constant (so-label "SO_LABEL") :optional t)                  ; freebsd
(constant (so-peerlabel "SO_PEERLABEL") :optional t)          ; freebsd
(constant (so-listenqlimit "SO_LISTENQLIMIT") :optional t)    ; freebsd
(constant (so-listenqlen "SO_LISTENQLEN") :optional t)        ; freebsd
(constant (so-listenincqlen "SO_LISTENINCQLEN") :optional t) ; freebsd

;; shutdown()
(constant (shut-rd "SHUT_RD"))
(constant (shut-wr "SHUT_WR"))
(constant (shut-rdwr "SHUT_RDWR"))

;; recvmsg/sendmsg()
(constant (msg-ctrunc "MSG_CTRUNC"))                 ; recvmsg
(constant (msg-dontroute "MSG_DONTROUTE"))           ;         sendmsg
(constant (msg-eor "MSG_EOR"))                       ; recvmsg sendmsg
(constant (msg-oob "MSG_OOB"))                       ; recvmsg sendmsg
(constant (msg-peek "MSG_PEEK"))                     ; recvmsg
(constant (msg-trunc "MSG_TRUNC"))                   ; recvmsg
(constant (msg-waitall "MSG_WAITALL"))               ; recvmsg
(constant (msg-dontwait "MSG_DONTWAIT"))             ; recvmsg sendmsg
(constant (msg-nosignal "MSG_NOSIGNAL"))             ;         sendmsg
(constant (msg-errqueue "MSG_ERRQUEUE") :optional t) ; recvmsg
(constant (msg-more "MSG_MORE") :optional t)         ;         sendmsg
(constant (msg-confirm "MSG_CONFIRM") :optional t)   ; sendmsg sendmsg
(constant (msg-proxy "MSG_PROXY") :optional t)       ; 
(constant (msg-fin "MSG_FIN") :optional t)           ; 
(constant (msg-syn "MSG_SYN") :optional t)           ; 
(constant (msg-eof "MSG_EOF") :optional t)           ; 
(constant (msg-nbio "MSG_NBIO") :optional t)         ; 
(constant (msg-compat "MSG_COMPAT") :optional t)     ;

(cstruct msghdr "struct msghdr"
 (name       "msg_name"       :type :pointer)
 (namelen    "msg_namelen"    :type socklen)
 (iov        "msg_iov"        :type :pointer)
 (iovlen     "msg_iovlen"     :type size)
 (control    "msg_control"    :type :pointer)
 (controllen "msg_controllen" :type socklen)
 (flags      "msg_flags"      :type :int))

(cstruct cmsghdr "struct cmsghdr"
 (len   "cmsg_len"   :type socklen)
 (level "cmsg_level" :type :int)
 (type  "cmsg_type"  :type :int))

(constant (cmgroup-max "CMGROUP_MAX") :optional t)
#+freebsd
(cstruct cmsgcred "struct cmsgcred"
 (pid     "cmcred_pid"     :type pid)
 (uid     "cmcred_uid"     :type uid)
 (euid    "cmcred_euid"    :type uid)
 (gid     "cmcred_gid"     :type gid)
 (ngroups "cmcred_ngroups" :type :short)
 (groups  "cmcred_groups"  :type gid :count :auto))

(constant (scm-rights "SCM_RIGHTS"))
(constant (scm-credentials "SCM_CREDENTIALS") :optional t)

#+linux
(cstruct ucred "struct ucred"
  "Socket credential messages."
  (pid "pid" :type pid)
  (uid "uid" :type uid)
  (gid "gid" :type gid))

#+freebsd
(cstruct sockcred "struct sockcred"
 (uid     "sc_uid"     :type uid)
 (euid    "sc_euid"    :type uid)
 (gid     "sc_gid"     :type gid)
 (egid    "sc_egid"    :type gid)
 (ngroups "sc_ngroups" :type :int)
 (groups  "sc_groups"  :type gid :count :auto))

(cstruct linger "struct linger"
 "SO_LINGER manipulation record."
 (onoff  "l_onoff"  :type :int)
 (linger "l_linger" :type :int))

#+freebsd
(cstruct accept-filter-arg "struct accept_filter_arg" 
 (name "af_name" :type :uint8 :count :auto)
 (arg  "af_arg"  :type :uint8 :count :auto))

;;
;; from sys/un.h
;;
(cstruct sockaddr-un "struct sockaddr_un"
 "A UNIX-domain socket address."
 (family "sun_family" :type sa-family)
 (path   "sun_path"   :type :uint8 :count :auto))

#+freebsd (constant (local-peercred "LOCAL_PEERCRED"))
#+freebsd (constant (local-creds "LOCAL_CREDS"))
#+freebsd (constant (local-connwait "LOCAL_CONNWAIT"))

;;
;; from netinet/in.h
;;
(ctype in-port "in_port_t")
(ctype in-addr "in_addr_t")

(cstruct sockaddr-in "struct sockaddr_in"
 "An IPv4 socket address."
 (family "sin_family" :type sa-family)
 (port   "sin_port"   :type in-port)
 (addr   "sin_addr"   :type in-addr))

(cunion in6-addr "struct in6_addr"
 "An IPv6 address."
 (addr8  "s6_addr"   :type :uint8  :count :auto)
 (addr16 "s6_addr16" :type :uint16 :count :auto)
 (addr32 "s6_addr32" :type :uint32 :count :auto))

(cstruct sockaddr-in6 "struct sockaddr_in6"
 "An IPv6 socket address."
 (family   "sin6_family"   :type sa-family)
 (port     "sin6_port"     :type in-port)
 (flowinfo "sin6_flowinfo" :type :uint32)
 (addr     "sin6_addr"     :type in6-addr)
 (scope-id "sin6_scope_id" :type :uint32))

(constant (inaddr-any "INADDR_ANY"))
(constant (inaddr-broadcast "INADDR_BROADCAST"))
(constant (inaddr-none "INADDR_NONE"))
(constant (in-loopbacknet "IN_LOOPBACKNET"))
(constant (inaddr-loopback "INADDR_LOOPBACK"))
(constant (inaddr-unspec-group "INADDR_UNSPEC_GROUP"))
(constant (inaddr-allhosts-group "INADDR_ALLHOSTS_GROUP"))
(constant (inaddr-allrtrs-group "INADDR_ALLRTRS_GROUP"))
(constant (inaddr-max-local-group "INADDR_MAX_LOCAL_GROUP"))
 
(constant (inet-addrstrlen "INET_ADDRSTRLEN"))
(constant (inet6-addrstrlen "INET6_ADDRSTRLEN"))

(constant (ipv6-join-group "IPV6_JOIN_GROUP"))
(constant (ipv6-leave-group "IPV6_LEAVE_GROUP"))
(constant (ipv6-multicast-hops "IPV6_MULTICAST_HOPS"))
(constant (ipv6-multicast-if "IPV6_MULTICAST_IF"))
(constant (ipv6-multicast-loop "IPV6_MULTICAST_LOOP"))
(constant (ipv6-unicast-hops "IPV6_UNICAST_HOPS"))
(constant (ipv6-v6only "IPV6_V6ONLY"))

;;
;; from netinet/tcp.h
;;
(constant (tcp-nodelay "TCP_NODELAY"))
(constant (tcp-maxseg "TCP_MAXSEG"))
#+linux
(constant (tcp-cork "TCP_CORK"))
(constant (tcp-keepidle "TCP_KEEPIDLE") :optional t)
(constant (tcp-keepintvl "TCP_KEEPINTVL") :optional t)
(constant (tcp-keepcnt "TCP_KEEPCNT") :optional t)
(constant (tcp-syncnt "TCP_SYNCNT") :optional t)
(constant (tcp-linger2 "TCP_LINGER2") :optional t)
(constant (tcp-defer-accept "TCP_DEFER_ACCEPT") :optional t)
(constant (tcp-window-clamp "TCP_WINDOW_CLAMP") :optional t)
(constant (tcp-info "TCP_INFO"))
(constant (tcp-quickack "TCP_QUICKACK") :optional t)

#+linux
(cenum connstates
 ((:tcp-established "TCP_ESTABLISHED"))
 ((:tcp-syn-sent "TCP_SYN_SENT"))
 ((:tcp-syn-recv "TCP_SYN_RECV"))
 ((:tcp-fin-wait1 "TCP_FIN_WAIT1"))
 ((:tcp-fin-wait2 "TCP_FIN_WAIT2"))
 ((:tcp-time-wait "TCP_TIME_WAIT"))
 ((:tcp-close "TCP_CLOSE"))
 ((:tcp-close-wait "TCP_CLOSE_WAIT"))
 ((:tcp-last-ack "TCP_LAST_ACK"))
 ((:tcp-listen "TCP_LISTEN"))
 ((:tcp-closing "TCP_CLOSING")))


;;
;; from netdb.h
;;

(constant (ipport-reserved "IPPORT_RESERVED"))

(cstruct addrinfo "struct addrinfo"
 (flags     "ai_flags"     :type :int)
 (family    "ai_family"    :type :int)
 (socktype  "ai_socktype"  :type :int)
 (protocol  "ai_protocol"  :type :int)
 (addrlen   "ai_addrlen"   :type socklen)
 (addr      "ai_addr"      :type :pointer)
 (canonname "ai_canonname" :type :pointer)
 (next      "ai_next"      :type :pointer))

;; addrinfo flags
(constant (ai-passive "AI_PASSIVE"))
(constant (ai-canonname "AI_CANONNAME"))
(constant (ai-numerichost "AI_NUMERICHOST"))
(constant (ai-numericserv "AI_NUMERICSERV"))
(constant (ai-v4mapped "AI_V4MAPPED"))
(constant (ai-v4mapped-cfg "AI_V4MAPPED_CFG") :optional t) ; freebsd
(constant (ai-all "AI_ALL"))
(constant (ai-addrconfig "AI_ADDRCONFIG"))

(constant (ni-maxhost "NI_MAXHOST"))
(constant (ni-maxserv "NI_MAXSERV"))

;; nameinfo flags
(constant (ni-nofqdn "NI_NOFQDN"))
(constant (ni-numerichost "NI_NUMERICHOST"))
(constant (ni-namereqd "NI_NAMEREQD"))
(constant (ni-numericserv "NI_NUMERICSERV"))
(constant (ni-numericscope "NI_NUMERICSCOPE") :optional t)
(constant (ni-dgram "NI_DGRAM"))

;; error codes
(constant (netdb-success "NETDB_SUCCESS"))
(constant (netdb-internal "NETDB_INTERNAL"))
(constantenum addrinfo-errors
 ((:netdb-success "NETDB_SUCCESS"))
 ((:eai-addrfamily "EAI_ADDRFAMILY") :optional t) ; linux
 ((:eai-again "EAI_AGAIN"))
 ((:eai-badflags "EAI_BADFLAGS"))
 ((:eai-fail "EAI_FAIL"))
 ((:eai-family "EAI_FAMILY"))
 ((:eai-memory "EAI_MEMORY"))
 ((:eai-noname "EAI_NONAME"))
 ((:eai-nodata "EAI_NODATA") :optional t)         ; linux
 ((:eai-service "EAI_SERVICE"))
 ((:eai-socktype "EAI_SOCKTYPE"))
 ((:eai-system "EAI_SYSTEM"))
 ((:eai-badhints "EAI_BADHINTS") :optional t)     ; freebsd
 ((:eai-protocol "EAI_PROTOCOL") :optional t)     ; freebsd
 ((:eai-max "EAI_MAX") :optional t)               ; freebsd
 ((:eai-overflow "EAI_OVERFLOW") :optional t))    ; linux

(cstruct protoent "struct protoent"
 (name    "p_name"    :type :string)
 (aliases "p_aliases" :type :pointer)
 (proto   "p_proto"   :type :int))


;;
;; from net/if.h
;;
(cstruct if-nameindex "struct if_nameindex" 
 (index "if_index" :type :unsigned-int)
 (name  "if_name"  :type :string))

(constant (ifnamesize "IF_NAMESIZE"))
(constant (ifnamsiz "IFNAMSIZ"))


;;
;; from time.h
;;
(ctype time "time_t")
(ctype suseconds "suseconds_t")
(ctype clockid "clockid_t")

(cstruct timespec "struct timespec"
 "UNIX time specification in seconds and nanoseconds."
 (sec  "tv_sec"  :type time)
 (nsec "tv_nsec" :type :long))

(constant (clock-monotonic "CLOCK_MONOTONIC"))
(constant (clock-realtime "CLOCK_REALTIME"))


;;
;; from sys/stat.h
;;
(ctype dev "dev_t")
(ctype ino "ino_t")
(ctype nlink "nlink_t")
(ctype blksize "blksize_t")
(ctype blkcnt "blkcnt_t")

(cstruct stat "struct stat"
 (dev     "st_dev"     :type #-mips dev #+mips :unsigned-long)
 (ino     "st_ino"     :type ino)
 (mode    "st_mode"    :type mode)
 (nlink   "st_nlink"   :type nlink)
 (uid     "st_uid"     :type uid)
 (gid     "st_gid"     :type gid)
 (rdev    "st_rdev"    :type #-mips dev #+mips :unsigned-long)
 (size    "st_size"    :type off)
 (blksize "st_blksize" :type blkcnt)
 (blocks  "st_blocks"  :type blksize)
 (atime   "st_atime"   :type time)
 (mtime   "st_mtime"   :type time)
 (ctime   "st_ctime"   :type time))

;;
;; from sys/select.h
;;
(constant (fd-setsize "FD_SETSIZE"))

(cstruct fd-set "fd_set"
 (bits "fds_bits" :type :uint8 :count :auto))

(cstruct timeval "struct timeval"
 "UNIX time specification in seconds and microseconds."
 (sec  "tv_sec"  :type time)
 (usec "tv_usec" :type suseconds))


;;
;; from syslog.h
;;

;; options
(constant (log-cons "LOG_CONS"))
(constant (log-ndelay "LOG_NDELAY"))
(constant (log-perror "LOG_PERROR"))
(constant (log-pid "LOG_PID"))

;; priority
(constantenum syslog-priorities
 ((:emerg "LOG_EMERG"))
 ((:alert "LOG_ALERT"))
 ((:crit "LOG_CRIT"))
 ((:err "LOG_ERR"))
 ((:warning "LOG_WARNING"))
 ((:notice "LOG_NOTICE"))
 ((:info "LOG_INFO"))
 ((:debug "LOG_DEBUG")))

;; facilities
(constantenum syslog-facilities
 ((:auth "LOG_AUTH"))
 ((:authpriv "LOG_AUTHPRIV"))
 ((:cron "LOG_CRON"))
 ((:daemon "LOG_DAEMON"))
 ((:ftp "LOG_FTP"))
 ((:kern "LOG_KERN"))
 ((:lpr "LOG_LPR"))
 ((:mail "LOG_MAIL"))
 ((:news "LOG_NEWS"))
 ((:security "LOG_SECURITY") :optional t)       ; freebsd
 ((:syslog "LOG_SYSLOG"))
 ((:user "LOG_USER"))
 ((:uucp "LOG_UUCP"))
 ((:local0 "LOG_LOCAL0"))
 ((:local1 "LOG_LOCAL1"))
 ((:local2 "LOG_LOCAL2"))
 ((:local3 "LOG_LOCAL3"))
 ((:local4 "LOG_LOCAL4"))
 ((:local5 "LOG_LOCAL5"))
 ((:local6 "LOG_LOCAL6"))
 ((:local7 "LOG_LOCAL7")))


;;
;; from sys/resource.h
;;

(ctype rlim "rlim_t")
(ctype id "id_t")

(cstruct rlimit "struct rlimit"
 (cur "rlim_cur" :type rlim)
 (max "rlim_max" :type rlim))

(cstruct rusage "struct rusage"
 (utime    "ru_utime"    :type timeval)
 (stime    "ru_stime"    :type timeval)
 (maxrss   "ru_maxrss"   :type :long)
 (ixrss    "ru_ixrss"    :type :long)
 (idrss    "ru_idrss"    :type :long)
 (isrss    "ru_isrss"    :type :long)
 (minflt   "ru_minflt"   :type :long)
 (majflt   "ru_majflt"   :type :long)
 (nswap    "ru_nswap"    :type :long)
 (inblock  "ru_inblock"  :type :long)
 (oublock  "ru_oublock"  :type :long)
 (msgsnd   "ru_msgsnd"   :type :long)
 (msgrcv   "ru_msgrcv"   :type :long)
 (nsignals "ru_nsignals" :type :long)
 (nvcsw    "ru_nvcsw"    :type :long)
 (nivcsw   "ru_nivcsw"   :type :long))

(constant (prio-process "PRIO_PROCESS"))
(constant (prio-pgrp "PRIO_PGRP"))
(constant (prio-user "PRIO_USER"))

(constant (rlim-infinity "RLIM_INFINITY"))
#+linux
(constant (rlim-saved-max "RLIM_SAVED_MAX"))
#+linux
(constant (rlim-saved-cur "RLIM_SAVED_CUR"))

(constant (rusage-self "RUSAGE_SELF"))
(constant (rusage-children "RUSAGE_CHILDREN"))

(constant (rlimit-as "RLIMIT_AS"))
(constant (rlimit-core "RLIMIT_CORE"))
(constant (rlimit-cpu "RLIMIT_CPU"))
(constant (rlimit-data "RLIMIT_DATA"))
(constant (rlimit-fsize "RLIMIT_FSIZE"))
#+linux
(constant (rlimit-locks "RLIMIT_LOCKS"))
(constant (rlimit-memlock "RLIMIT_MEMLOCK"))
#+linux
(constant (rlimit-msgqueue "RLIMIT_MSGQUEUE"))
#+linux
(constant (rlimit-nice "RLIMIT_NICE"))
#+linux
(constant (rlimit-nlimits "RLIMIT_NLIMITS"))
(constant (rlimit-nofile "RLIMIT_NOFILE"))
(constant (rlimit-nproc "RLIMIT_NPROC"))
(constant (rlimit-rss "RLIMIT_RSS"))
#+linux
(constant (rlimit-rtprio "RLIMIT_RTPRIO"))
#+linux
(constant (rlimit-sigpending "RLIMIT_SIGPENDING"))
#+freebsd
(constant (rlimit-sbsize "RLIMIT_SBSIZE"))
(constant (rlimit-stack "RLIMIT_STACK"))


;;
;; from sys/utsname.h
;;

(cstruct utsname "struct utsname"
 (sysname  "sysname"  :type :char)
 (nodename "nodename" :type :char)
 (release  "release"  :type :char)
 (version  "version"  :type :char)
 (machine  "machine"  :type :char))


;;
;; from sys/statvfs.h
;;

(ctype fsblkcnt "fsblkcnt_t")
(ctype fsfilcnt "fsfilcnt_t")

(cstruct statvfs "struct statvfs"
 (bsize   "f_bsize"   :type :unsigned-long)
 (frsize  "f_frsize"  :type :unsigned-long)
 (blocks  "f_blocks"  :type fsblkcnt)
 (bfree   "f_bfree"   :type fsblkcnt)
 (bavail  "f_bavail"  :type fsblkcnt)
 (files   "f_files"   :type fsfilcnt)
 (ffree   "f_ffree"   :type fsfilcnt)
 (favail  "f_favail"  :type fsfilcnt)
 (fsig    "f_fsid"    :type :unsigned-long)
 (flag    "f_flag"    :type :unsigned-long)
 (namemax "f_namemax" :type :unsigned-long))

(constant (st-rdonly "ST_RDONLY"))
(constant (st-nosuid "ST_NOSUID"))
#+linux
(constant (st-nodev "ST_NODEV"))
#+linux
(constant (st-noexec "ST_NOEXEC"))
#+linux
(constant (st-synchronous "ST_SYNCHRONOUS"))
#+linux
(constant (st-mandlock "ST_MANDLOCK"))
#+linux
(constant (st-write "ST_WRITE"))
#+linux
(constant (st-append "ST_APPEND"))
#+linux
(constant (st-immutable "ST_IMMUTABLE"))
#+linux
(constant (st-noatime "ST_NOATIME"))
#+linux
(constant (st-nodiratime "ST_NODIRATIME"))


;;
;; from sys/poll.h
;;
(ctype nfds "nfds_t")
(cstruct pollfd "struct pollfd"
 "Poll file descriptor activity specification structure."
 (fd      "fd"      :type :int)
 (events  "events"  :type :short)
 (revents "revents" :type :short))

(constant (pollin "POLLIN"))
(constant (pollrdnorm "POLLRDNORM"))
(constant (pollrdband "POLLRDBAND"))
(constant (pollpri "POLLPRI"))
(constant (pollout "POLLOUT"))
(constant (pollwrnorm "POLLWRNORM"))
(constant (pollwrband "POLLWRBAND"))
(constant (pollerr "POLLERR"))
(constant (pollrdhup "POLLRDHUP"))
(constant (pollhup "POLLHUP"))
(constant (pollnval "POLLNVAL"))


;;
;; from sys/epoll.h
;;

#+linux
(cunion epoll-data "epoll_data_t"
 (ptr "ptr" :type :pointer)
 (fd  "fd"  :type :int)
 (u32 "u32" :type :uint32)
 (u64 "u64" :type :uint64))

#+linux
(cstruct epoll-event "struct epoll_event"
 (events "events" :type :uint32)
 (data   "data"   :type epoll-data))

#+linux (constant (epollin "EPOLLIN"))
#+linux (constant (epollrdnorm "EPOLLRDNORM"))
#+linux (constant (epollrdband "EPOLLRDBAND"))
#+linux (constant (epollpri "EPOLLPRI"))
#+linux (constant (epollout "EPOLLOUT"))
#+linux (constant (epollwrnorm "EPOLLWRNORM"))
#+linux (constant (epollwrband "EPOLLWRBAND"))
#+linux (constant (epollerr "EPOLLERR"))
#+linux (constant (epollhup "EPOLLHUP"))
#+linux (constant (epollmsg "EPOLLMSG"))
#+linux (constant (epolloneshot "EPOLLONESHOT"))
#+linux (constant (epollet "EPOLLET"))

#+linux (constant (epoll-ctl-add "EPOLL_CTL_ADD"))
#+linux (constant (epoll-ctl-del "EPOLL_CTL_DEL"))
#+linux (constant (epoll-ctl-mod "EPOLL_CTL_MOD"))


;;
;; from sys/event.h
;;

#+freebsd (ctype intptr "intptr_t")
#+freebsd (ctype uintptr "uintptr_t")

#+freebsd
(cstruct kevent "struct kevent"
 (ident  "ident"  :type uintptr)
 (filter "filter" :type :short)
 (flags  "flags"  :type :unsigned-short)
 (fflags "fflags" :type :unsigned-int)
 (data   "data"   :type intptr)
 (udata  "udata"  :type :pointer))

;; kevent() flags
#+freebsd (constant (ev-add "EV_ADD"))
#+freebsd (constant (ev-enable "EV_ENABLE"))
#+freebsd (constant (ev-disable "EV_DISABLE"))
#+freebsd (constant (ev-delete "EV_DELETE"))
#+freebsd (constant (ev-oneshot "EV_ONESHOT"))
#+freebsd (constant (ev-clear "EV_CLEAR"))
#+freebsd (constant (ev-eof "EV_EOF"))
#+freebsd (constant (ev-error "EV_ERROR"))

;; kevent() filter flags
#+freebsd (constant (evfilt-read "EVFILT_READ"))
#+freebsd (constant (evfilt-write "EVFILT_WRITE"))
#+freebsd (constant (evfilt-aio "EVFILT_AIO"))
#+freebsd (constant (evfilt-vnode "EVFILT_VNODE"))
#+freebsd (constant (evfilt-proc "EVFILT_PROC"))
#+freebsd (constant (evfilt-signal "EVFILT_SIGNAL"))
#+freebsd (constant (evfilt-timer "EVFILT_TIMER"))
#+freebsd (constant (evfilt-netdev "EVFILT_NETDEV"))

;; EVFILT_VNODE options
#+freebsd (constant (note-delete "NOTE_DELETE"))
#+freebsd (constant (note-write "NOTE_WRITE"))
#+freebsd (constant (note-extend "NOTE_EXTEND"))
#+freebsd (constant (note-attrib "NOTE_ATTRIB"))
#+freebsd (constant (note-link "NOTE_LINK"))
#+freebsd (constant (note-rename "NOTE_RENAME"))
#+freebsd (constant (note-revoke "NOTE_REVOKE"))

;; EVFILT_PROC options
#+freebsd (constant (note-exit "NOTE_EXIT"))
#+freebsd (constant (note-fork "NOTE_FORK"))
#+freebsd (constant (note-exec "NOTE_EXEC"))
#+freebsd (constant (note-track "NOTE_TRACK"))
#+freebsd (constant (note-trackerr "NOTE_TRACKERR"))

;; EVFILT_NETDEV options
#+freebsd (constant (note-linkup "NOTE_LINKUP"))
#+freebsd (constant (note-linkdown "NOTE_LINKDOWN"))
#+freebsd (constant (note-linkinv "NOTE_LINKINV"))


;;
;; from unistd.h
;;
(ctype useconds "useconds_t")


;;
;; from pwd.h
;;
(cstruct passwd "struct passwd"
 (name   "pw_name"   :type :string)
 (passwd "pw_passwd" :type :string)
 (uid    "pw_uid"    :type uid)
 (gid    "pw_gid"    :type gid)
 (gecos  "pw_gecos"  :type :string)
 (dir    "pw_dir"    :type :string)
 (shell  "pw_shell"  :type :string))


;;
;; from grp.h
;;
(cstruct group "struct group"
 (name   "gr_name"   :type :string)
 (passwd "gr_passwd" :type :string)
 (gid    "gr_gid"    :type gid))
; FIXME What about gr_mem?  Kinda the whole point?


;;
;; from dirent.h
;;

;; Apparently POSIX 1003.1-2001 (according to linux manpages) only
;; requires d_name.  Sigh.  I guess we should assemble some decent
;; wrapper functions.  No, struct-members can't be optional at this
;; point.
(cstruct dirent "struct dirent"
 ;; POSIX actually requires this to be d_ino
 (fileno "d_fileno" :type #-freebsd ino #+freebsd :uint32)
 (type   "d_type"   :type :uint8)
 (name   "d_name"   :type :uint8 :count :auto))

;; filetypes set in d_type slot of struct dirent
(constant (dt-unknown "DT_UNKNOWN"))
(constant (dt-fifo "DT_FIFO"))
(constant (dt-chr "DT_CHR"))
(constant (dt-dir "DT_DIR"))
(constant (dt-blk "DT_BLK"))
(constant (dt-reg "DT_REG"))
(constant (dt-lnk "DT_LNK"))
(constant (dt-sock "DT_SOCK"))
(constant (dt-wht "DT_WHT"))

;; THE END
